@@ -4,26 +4,28 @@ function(sgp_object,
          years,
          content_areas,
          grades,
-         sgp.config,
          sgp.percentiles=TRUE, 
          sgp.projections=TRUE,
          sgp.projections.lagged=TRUE,
+         sgp.percentiles.baseline=FALSE,
          simulate.sgps=TRUE,
          goodness.of.fit.print=TRUE,
+         sgp.config,
+         sgp.baseline.config, 
          parallel.config,
          ...) {
 
 	started.at <- proc.time()
 	message(paste("Started analyzeSGP", date()))
 
-        ### Create state (if missing) from sgp_object (if possible)
+	### Create state (if missing) from sgp_object (if possible)
 
-        if (missing(state)) {
-                tmp.name <- gsub("_", " ", deparse(substitute(sgp_object)))
-                if (any(sapply(c(state.name, "Demonstration"), function(x) regexpr(x, tmp.name)))==1) {
-                        state <- c(state.abb, "DEMO")[which(sapply(c(state.name, "Demonstration"), function(x) regexpr(x, tmp.name))==1)]
-                }
-        }
+	if (missing(state)) {
+		tmp.name <- gsub("_", " ", deparse(substitute(sgp_object)))
+		if (any(sapply(c(state.name, "Demonstration"), function(x) regexpr(x, tmp.name)))==1) {
+			state <- c(state.abb, "DEMO")[which(sapply(c(state.name, "Demonstration"), function(x) regexpr(x, tmp.name))==1)]
+		}
+	}
 
 	## Function to return sgp.config based upon a supplied year and content_area
 
@@ -31,28 +33,24 @@ function(sgp_object,
 		tmp.data <- sgp_object@Data[J("VALID_CASE", content_area), c("YEAR", "GRADE"), with=FALSE]
 		.sgp.panel.years <- sort(unique(tmp.data$YEAR))[1:which(sort(unique(tmp.data$YEAR)) == year)]
 		.sgp.content.areas <- rep(content_area, length(.sgp.panel.years))
-		.tmp.sgp.grade.sequences <- lapply(grades, function(x) tail(seq(min(tmp.data$GRADE, na.rm=TRUE), x), length(.sgp.panel.years)))
-		.tmp.table <- table(tmp.data$GRADE, tmp.data$YEAR)
-		.sgp.grade.sequences <- sapply(.tmp.sgp.grade.sequences, 
-			function(x) tail(x, sum(diag(.tmp.table[as.character(x), as.character(tail(.sgp.panel.years, length(x)))])!=0))) 
+		.sgp.grade.sequences <- lapply(grades[-1], function(x) tail(grades[grades <= x], length(unique(tmp.data$YEAR))-1))
+		for (g in seq_along(.sgp.grade.sequences)) {
+			.sgp.grade.sequences[[g]]<-.sgp.grade.sequences[[g]][tail(.sgp.grade.sequences[[g]],1)-.sgp.grade.sequences[[g]] <= length(.sgp.panel.years)-1]
+		}
 		list(sgp.content.areas=.sgp.content.areas, sgp.panel.years=.sgp.panel.years, sgp.grade.sequences=.sgp.grade.sequences) 
 	}
 
 	.analyzeSGP_Internal <- function(sgp.iter,
 		sgp.percentiles_Internal=sgp.percentiles,
+		sgp.percentiles.baseline_Internal= sgp.percentiles.baseline,
 		sgp.projections_Internal=sgp.projections,
-		sgp.projections.lagged_Internal=sgp.projections.lagged) {
+		sgp.projections.lagged_Internal=sgp.projections.lagged,
+		sgp.baseline.config=sgp.baseline.config) {
 
-		tmp_sgp_object <- list()
-
-		if (!is.null(sgp_object@SGP)) {
-			for (i in names(sgp_object@SGP)) {
-				tmp_sgp_object[[i]] <- sgp_object@SGP[[i]]
-			}
-		}
+		tmp_sgp_object <- .mergeSGP(list(), sgp_object@SGP)
 
 		tmp_sgp_object[["Panel_Data"]] <- 
-		as.data.frame(reshape(sgp_object@Data[J("VALID_CASE", sgp.iter[["sgp.content.areas"]], sgp.iter[["sgp.panel.years"]]), mult="all"], #VA-diff subjects, MA + AlgI no [1] in sgp.iter[["sgp.content.areas"]][1]
+		as.data.frame(reshape(sgp_object@Data[J("VALID_CASE", sgp.iter[["sgp.content.areas"]], sgp.iter[["sgp.panel.years"]]), mult="all"],
 			idvar="ID",
 			timevar="YEAR",
 			drop=names(sgp_object@Data)[!names(sgp_object@Data) %in% c("ID", "GRADE", "SCALE_SCORE", "YEAR")],
@@ -106,7 +104,7 @@ function(sgp_object,
 					use.my.coefficient.matrices=list(my.year=tail(sgp.iter[["sgp.panel.years"]], 1), my.subject=tail(sgp.iter[["sgp.content.areas"]], 1)), 
 					use.my.knots.boundaries=list(my.year=tail(sgp.iter[["sgp.panel.years"]], 1), my.subject=tail(sgp.iter[["sgp.content.areas"]], 1)), 
 					performance.level.cutscores=state,
-					max.forward.progression=3,
+					max.forward.progression.years=3,
 					percentile.trajectory.values=c(1, stateData[[state]][["Growth"]][["Cutscores"]][["Cuts"]], 99),
 					panel.data.vnames=sgp.vnames,
 					grade.progression=k,
@@ -133,6 +131,79 @@ function(sgp_object,
 					...)
 			}
 		} ## END sgp.projections.lagged_Internal
+
+		## sgp.percentiles.baseline_Internal
+		
+		if (sgp.percentiles.baseline_Internal) {
+			sgp.vnames <- c("ID", paste("GRADE", sgp.iter[["sgp.panel.years"]], sep="."), 
+				paste("SCALE_SCORE", sgp.iter[["sgp.panel.years"]], sep="."))
+				
+			tmp_sgp_object <- .mergeSGP(tmp_sgp_object, stateData[[state]][["Baseline_splineMatrix"]])
+			
+			if (is.null(tmp_sgp_object[["Coefficient_Matrices"]][[paste(toupper(tail(sgp.iter[["sgp.content.areas"]], 1)), ".BASELINE", sep="")]])) {
+				if (missing(sgp.baseline.config)){
+					.content_areas <- .years <- .grades <- .sgp.grade.sequences <- list()
+					.content_areas <- tail(sgp.iter[["sgp.content.areas"]], 1)
+					.years <- sort(unique(sgp_object@Data[J("VALID_CASE", .content_areas)][["YEAR"]]))
+					.grades <- sort(unique(sgp_object@Data[J("VALID_CASE", .content_areas)][["GRADE"]]))
+					.baseline.max.order <- length(.years)-2
+					.sgp.grade.sequences <- lapply(.grades[-1], function(x) tail(.grades[.grades <= x], (.baseline.max.order+1)))
+					for (g in seq_along(.sgp.grade.sequences)) {
+						.sgp.grade.sequences[[g]]<-.sgp.grade.sequences[[g]][tail(.sgp.grade.sequences[[g]],1)-.sgp.grade.sequences[[g]]<=.baseline.max.order]
+					}
+					sgp.baseline.config <- list(baseline.content.areas=.content_areas, baseline.panel.years=.years, 
+						baseline.max.order=.baseline.max.order, baseline.grade.sequences=.sgp.grade.sequences)
+				}
+			if (!identical(names(sgp.baseline.config), c("baseline.content.areas", "baseline.panel.years", "baseline.max.order", "baseline.grade.sequences"))) {
+				 stop("Please specify an appropriate list of SGP function labels (sgp.baseline.config).	See help page for details.")
+			}
+				message("\n\tCreating baseline matrices...\n")
+
+				sgp_object <- baselineSGP(sgp_object,
+						state=state,
+						years=sgp.baseline.config[["baseline.panel.years"]],
+						content_areas=sgp.baseline.config[["baseline.content.areas"]],
+						grade.sequences=sgp.baseline.config[["baseline.grade.sequences"]],
+						baseline.max.order=sgp.baseline.config[["baseline.max.order"]],
+						goodness.of.fit=FALSE)
+				
+				Baseline_Matrices <- NULL # Avoid R CMD check NOTE about visible binding for global variable
+				eval(parse(text=paste("Baseline_Matrices <- list(", sgp.baseline.config[["baseline.content.areas"]],
+					".BASELINE = sgp_object@SGP[['Coefficient_Matrices']][['", sgp.baseline.config[["baseline.content.areas"]], ".BASELINE']])", sep="")))
+				save(Baseline_Matrices, file=paste(state, "_", sgp.baseline.config[["baseline.content.areas"]], "_",  "Baseline_Matrices.Rdata", sep=""), compress=TRUE) 
+
+				tmp_sgp_object <- .mergeSGP(tmp_sgp_object, sgp_object@SGP)
+				message("\n\tDone computing baseline matrices...\n")
+			}
+				
+			mtx.names <- names(tmp_sgp_object[["Coefficient_Matrices"]][[paste(toupper(tail(sgp.iter[["sgp.content.areas"]], 1)), ".BASELINE", sep="")]])
+
+			for (k in sgp.iter[["sgp.grade.sequences"]]) {
+				
+				max.order.mtx <- mtx.names[[max(grep(paste("qrmatrix_", tail(k, 1), sep=""), mtx.names))]]
+				max.order <- as.numeric(strsplit(max.order.mtx, "_")[[1]][3])
+				
+				if (length(sgp.iter[["sgp.panel.years"]])-1 < max.order) max.order <- length(sgp.iter[["sgp.panel.years"]])-1			
+
+				if (sum(diff(tail(k, 1+max.order))) >= length(sgp.iter[["sgp.panel.years"]])) {		# deals with 'holes' ... again
+					base.gp <- k[tail(k, 1)-k <= max.order]
+					max.order <- length(base.gp) - 1
+				}	else base.gp <- tail(k, 1+max.order)
+
+				tmp_sgp_object <- studentGrowthPercentiles(
+										panel.data = tmp_sgp_object,
+										sgp.labels = list(my.year = tail(sgp.iter[["sgp.panel.years"]], 1), 
+											my.subject = tail(sgp.iter[["sgp.content.areas"]], 1), my.extra.label="BASELINE"),
+										use.my.knots.boundaries = state,
+										use.my.coefficient.matrices = list(my.year = "BASELINE", my.subject = tail(sgp.iter[["sgp.content.areas"]], 1)),
+										growth.levels = state,
+										panel.data.vnames = sgp.vnames,
+										grade.progression = base.gp,
+										num.prior = max.order,
+										goodness.of.fit=FALSE,
+										...)
+			} ## END k loop
+		} ## END if sgp.percentiles.baseline_Internal
     	return(tmp_sgp_object)
 	} ## END .analyzeSGP_Internal
 
@@ -141,7 +212,7 @@ function(sgp_object,
 			list_1[[j]] <- c(list_1[[j]], list_2[[j]])[!duplicated(names(c(list_1[[j]], list_2[[j]])))]
 		}
 		for (j in c("SGPercentiles", "SGProjections", "Simulated_SGPs")) {
-			if (all(names(list_2[[j]]) %in% names(list_1[[j]])) & !identical(list_1[[j]], list_2[[j]])) { #all(), not identical
+			if (all(names(list_2[[j]]) %in% names(list_1[[j]])) & !identical(list_1[[j]], list_2[[j]])) {
 				for (k in names(list_1[[j]])) {
 					list_1[[j]][[k]] <- rbind.fill(list_1[[j]][[k]], list_2[[j]][[k]][!list_2[[j]][[k]][["ID"]] %in% list_1[[j]][[k]][["ID"]],]);gc()
 				}
@@ -179,7 +250,7 @@ function(sgp_object,
 		}
 		if (missing(years)) {
 			for (i in content_areas) {
-				tmp.years[[i]] <- sort(tail(unique(sgp_object@Data[J("VALID_CASE", content_areas)]$YEAR), -2), decreasing=TRUE)
+				tmp.years[[i]] <- sort(tail(unique(sgp_object@Data[J("VALID_CASE", i)][["YEAR"]]), -2), decreasing=TRUE)
 			}
 		} else {
 			for (i in content_areas) { 
@@ -189,7 +260,7 @@ function(sgp_object,
 		if (missing(grades)) {
 			for (i in content_areas) {
 				for (j in tmp.years[[i]]) {
-					tmp.grades[[paste(i,j,sep=".")]] <- tail(sort(unique(sgp_object@Data[J("VALID_CASE", i, j)]$GRADE)), -1)
+					tmp.grades[[paste(i,j,sep=".")]] <- sort(unique(sgp_object@Data[J("VALID_CASE", i, j)][["GRADE"]]))
 				}
 			}
 		} else {
@@ -208,45 +279,50 @@ function(sgp_object,
 
 	## studentGrowthPercentiles & studentGrowthProjections
 
-	if (sgp.percentiles | sgp.projections | sgp.projections.lagged) {
+	if (sgp.percentiles | sgp.projections | sgp.projections.lagged | sgp.percentiles.baseline) {
 		sgp.iter <- NULL ## To prevent R CMD check warning
 
 		if (missing(parallel.config)) {
-			tmp <- list()
 			for (i in 1:length(sgp.config)) {
-				tmp[[i]] <- .analyzeSGP_Internal(sgp.config[[i]])
+				sgp_object@SGP <- .mergeSGP(sgp_object@SGP, .analyzeSGP_Internal(sgp.config[[i]])) #simplified, and works with baseline matrix creation
 			}
-			for (s in 1:length(tmp)) {
-				sgp_object@SGP <- .mergeSGP(sgp_object@SGP,tmp[[s]])
-			}
-		} else {
+		}	else {
 		
-		if (toupper(parallel.config[["TYPE"]]) == "FOREACH") {
-			require(foreach)
-			foreach.options <-parallel.config[["OPTIONS"]] # works fine if NULL
-			sgp_object@SGP <- foreach(sgp.iter=iter(sgp.config), .packages="SGP", .combine=".mergeSGP", .inorder=FALSE,
-				.options.multicore=foreach.options, .options.mpi= foreach.options, .options.redis= foreach.options, .options.smp= foreach.options) %dopar% {
-					return(.analyzeSGP_Internal(sgp.iter))
+			if (toupper(parallel.config[["TYPE"]]) == "FOREACH") {
+				require(foreach)
+				if (!is.null(parallel.config[["WORKERS"]])) {
+					cores <- parallel.config[["WORKERS"]]
+				} else cores = getOption("cores")
+				foreach.options <-parallel.config[["OPTIONS"]] # works fine if NULL
+				sgp_object@SGP <- foreach(sgp.iter=iter(sgp.config), .packages="SGP", .combine=".mergeSGP", .inorder=FALSE,
+					.options.multicore=foreach.options, .options.mpi= foreach.options, .options.redis= foreach.options, .options.smp= foreach.options) %dopar% {
+						return(.analyzeSGP_Internal(sgp.iter))
+				}
+			}
+
+			if (toupper(parallel.config[["TYPE"]]) == "MULTICORE") {
+				require(multicore)
+				if (!is.null(parallel.config[["WORKERS"]])) {
+					cores <- parallel.config[["WORKERS"]]
+				} else cores = getOption("cores")
+				tmp <- mclapply(sgp.config, .analyzeSGP_Internal, mc.cores=cores, mc.preschedule = FALSE)
+				for (s in 1:length(tmp))	sgp_object@SGP <- .mergeSGP(sgp_object@SGP, tmp[[s]])
+			}
+
+			if (toupper(parallel.config[["TYPE"]]) == "SNOW") {
+				require(snow)
+				if (is.null(parallel.config[["CLUSTER.OBJECT"]])) cluster.object <- "cl"
+					else cluster.object <- parallel.config[["CLUSTER.OBJECT"]]
+				tmp <- parLapply(eval(parse(text=cluster.object)), sgp.config, .analyzeSGP_Internal)
+				for (s in 1:length(tmp))	sgp_object@SGP <- .mergeSGP(sgp_object@SGP, tmp[[s]])
+			}
+
+			if (!toupper(parallel.config[["TYPE"]]) %in% c("FOREACH", "MULTICORE", "SNOW")) {
+				stop("'TYPE' provided in parallel.config not recognized.  Currently 'FOREACH', 'MULTICORE' and 'SNOW' are supported.  See help pages for additional information.")
 			}
 		}
 
-		if (toupper(parallel.config[["TYPE"]]) == "MULTICORE") {
-			require(doMC)
-			tmp <- mclapply(sgp.config, .analyzeSGP_Internal, mc.preschedule = FALSE)
-			for (s in 1:length(tmp))	sgp_object@SGP <- .mergeSGP(sgp_object@SGP, tmp[[s]])
-		}
-
-		if (toupper(parallel.config[["TYPE"]]) == "SNOW") {
-			require(snow)
-			tmp <- parLapply(parallel.config[["CLUSTER"]], sgp.config, .analyzeSGP_Internal)
-			for (s in 1:length(tmp))	sgp_object@SGP <- .mergeSGP(sgp_object@SGP, tmp[[s]])
-		}
-
-		if (!toupper(parallel.config[["TYPE"]]) %in% c("FOREACH", "MULTICORE", "SNOW")) {
-			stop("'TYPE' provided in parallel.config not recognized.  Currently 'FOREACH', 'MULTICORE' and 'SNOW' are supported.  See help pages for additional information.")
-		}}
-
-	} ## END if (sgp.percentiles | sgp.projections | sgp.projections.lagged)
+	} ## END if (sgp.percentiles | sgp.projections | sgp.projections.lagged | sgp.percentiles.baseline)
 
 	if (goodness.of.fit.print) gof.print(sgp_object)
 
