@@ -53,6 +53,10 @@ arrow.color <- function(sgp){
           arrow.legend.color[findInterval(sgp, growth.level.cutscores)+1]
 }
 
+.year.increment <- function(year, increment) {
+      paste(as.numeric(unlist(strsplit(as.character(year), "_")))+increment, collapse="_")
+}
+
 interpolate.grades <- function(grades, data.year.span) {
 
             last.number <- function (x) {
@@ -91,57 +95,75 @@ interpolate.grades <- function(grades, data.year.span) {
               if (grades[1] == max(grades.reported.in.state)) {
                   year_span <- 5
                   temp.grades <- extend.grades(rev(grades))
-                  return (list(interp.df = data.frame(GRADE=temp.grades), year_span=year_span))
+                  return (list(interp.df = data.frame(GRADE=temp.grades), 
+				year_span=year_span, 
+				years=sapply(temp.grades-grades[1], function(x) .year.increment(Report_Parameters$Current_Year, x))))
               } else {
                   if (grades[1] == 10) {
                      year_span <- 4
                      temp.grades <- extend.grades(rev(grades))
-                     return (list(interp.df = data.frame(GRADE=temp.grades), year_span=year_span))
+                     return (list(interp.df = data.frame(GRADE=temp.grades), 
+				year_span=year_span, 
+				years=sapply(temp.grades-grades[1], function(x) .year.increment(Report_Parameters$Current_Year, x))))
                   }
 
                   if (grades[1] == 9) {
                      year_span <- 4
                      temp.grades <- extend.grades(c(rev(head(grades, -1)), 10))
-                     return (list(interp.df = data.frame(GRADE=temp.grades), year_span=year_span))
+                     return (list(interp.df = data.frame(GRADE=temp.grades), 
+				year_span=year_span, 
+				years=sapply(temp.grades-grades[1], function(x) .year.increment(Report_Parameters$Current_Year, x))))
                   }
              
                   if (grades[1] == 8) {
                      year_span <- 4
                      temp.grades <- extend.grades(c(rev(head(grades, -1)), 9))
-                     return (list(interp.df = data.frame(GRADE=temp.grades), year_span=year_span))
+                     return (list(interp.df = data.frame(GRADE=temp.grades), 
+				year_span=year_span, 
+				years=sapply(temp.grades-grades[1], function(x) .year.increment(Report_Parameters$Current_Year, x))))
                   }
 
                   if (grades[1] == 7) {
                      year_span <- 4
                      temp.grades <- extend.grades(c(rev(head(grades, -1)), 8))
-                     return (list(interp.df = data.frame(GRADE=temp.grades), year_span=year_span))
+                     return (list(interp.df = data.frame(GRADE=temp.grades), 
+				year_span=year_span, 
+				years=sapply(temp.grades-grades[1], function(x) .year.increment(Report_Parameters$Current_Year, x))))
                   }
 
                   if (grades[1] == 6) {
                      year_span <- 4
                      temp.grades <- extend.grades(c(rev(head(grades, -1)), 7))
-                     return (list(interp.df = data.frame(GRADE=temp.grades), year_span=year_span))
+                     return (list(interp.df = data.frame(GRADE=temp.grades), 
+				year_span=year_span, 
+				years=sapply(temp.grades-grades[1], function(x) .year.increment(Report_Parameters$Current_Year, x))))
                   }
 
                   if (grades[1] == 5) {
                      year_span <- max(min(last.scale.score, 4), 3)
                      grades <- head(grades, year_span)
                      temp.grades <- extend.grades(c(rev(grades), head(6:7, data.year.span-year_span)))
-                     return (list(interp.df = data.frame(GRADE=temp.grades), year_span=year_span))
+                     return (list(interp.df = data.frame(GRADE=temp.grades), 
+				year_span=year_span, 
+				years=sapply(temp.grades-grades[1], function(x) .year.increment(Report_Parameters$Current_Year, x))))
                   }
 
                   if (grades[1] == 4) {
                      year_span <- max(min(last.scale.score, 4), 2)
                      grades <- head(grades, year_span)
                      temp.grades <- extend.grades(c(rev(grades), head(5:7, data.year.span-year_span)))
-                     return (list(interp.df = data.frame(GRADE=temp.grades), year_span=year_span))
+                     return (list(interp.df = data.frame(GRADE=temp.grades), 
+				year_span=year_span, 
+				years=sapply(temp.grades-grades[1], function(x) .year.increment(Report_Parameters$Current_Year, x))))
                   }
 
                   if (grades[1] == 3) {
                      year_span <- max(min(last.scale.score, 4), 1)
                      grades <- head(grades, year_span)
                      temp.grades <- extend.grades(c(rev(grades), head(4:7, data.year.span-year_span)))
-                     return (list(interp.df = data.frame(GRADE=temp.grades), year_span=year_span))
+                     return (list(interp.df = data.frame(GRADE=temp.grades), 
+				year_span=year_span, 
+				years=sapply(temp.grades-grades[1], function(x) .year.increment(Report_Parameters$Current_Year, x))))
                   }
               } 
        }
@@ -377,7 +399,13 @@ pushViewport(subject.report.vp)
 pushViewport(growth.chart.vp)
 
 for (i in seq(number.achievement.level.regions-1)){
-    temp <- merge(cbind(temp_id=seq_len(nrow(grade.values$interp.df)), grade.values$interp.df), subset(Cutscores, CUTLEVEL==i), all.x=TRUE)
+    temp <- cbind(temp_id=seq_len(nrow(grade.values$interp.df)), grade.values$interp.df, YEAR=grade.values$years)
+    temp$YEAR[!unique(temp$YEAR) %in% unique(Cutscores$YEAR)] <- NA
+    if (any(!is.na(temp$YEAR))) {
+       tmp.last.observation <- max(which(!is.na(temp$YEAR)))
+       temp$YEAR[tmp.last.observation:length(temp$YEAR)] <- temp$YEAR[tmp.last.observation]
+    }
+    temp <- merge(temp, subset(Cutscores, CUTLEVEL==i), all.x=TRUE)
     temp <- temp[order(temp$temp_id),]$CUTSCORE
     temp[which(is.na(temp))] <- approx(temp, xout=which(is.na(temp)))$y
     assign(paste("level_", i, "_curve", sep=""), splinefun((low.year-1):(high.year+1), temp, method="mono"))
